/**
 * CLASS: Level (a modified version of the support class uploaded on the site)
 * AUTHOR: danpost (greenfoot.org username)
 * 
 * ABOUT:  This class is a to be instantiated as the initial world used for the title page.
 * The subclasses of this class create the different game play levels in the project and
 * this class contains the methods and fields that are common to all subclasses.  These are
 * mainly for level initialization and for changing levels.
 * 
 * Fields for the level display actor and the score actor used to be contained in this class.
 * However, with the introduction of my "control" classes, they are no longer needed here.
 */
import greenfoot.*;
import java.awt.Font;
import java.awt.Color;

public class Level extends World
{
    // class fields
    static Player player;
    // each subclass assigns its map to the following field for level initialization
    protected String[] map;
    
    /**
     * constructor of the title page world and initializer of static fields in project
     */
    public Level()
    {
        super(800, 600, 1);
        // initialize static fields
        TextImage.setDefaultSize(24);
        TextImage.setDefaultName("JasmineUPC Bold Italic");
        ScoreControl.setScore(0);
        LevelControl.setLevel(0);
        player = new Player();
        
        // create the title screen background image
        GreenfootImage bg = new GreenfootImage(6, 4);
        bg.setColor(Color.darkGray);
        bg.fill();
        bg.setColor(Color.red);
        bg.drawLine(0, 0, 5, 3);
        bg.drawLine(5, 0, 0, 3);
        setBackground(bg);
        
        // the title screen text
        Font font = TextImage.getFont("Curlz MT", 240);
        Color tColor = Color.orange.darker();
        drawCircText("COMBO", font, tColor, null, 1, false, 380, 360, 280, 190, 30);
        drawCircText("DEMO", font, tColor, null, 1, true, 420, 260, 250, 115, -30);
        font = TextImage.getFont("Curlz MT", 64);
        drawCircText("S U P P O R T", font, tColor, null, 1, false, 380, 330, 150, 180, 10);
        drawCircText("C L A S S", font, tColor, null, 1, true, 420, 250, 170, 105, -10);
        // the author text
        font = TextImage.getFont("Harlow Solid Italic", 48);
        BgInk ink = new BgInk();
        ink.setImage(new TextImage("by\ndanpost", font, Color.black, null).styled(1));
        addObject(ink, 396, 292);
        ink.turn(-30);
        ink.apply();
        GreenfootImage image = new GreenfootImage(180, 180);
        image.setColor(new Color(127, 127, 127, 144));
        image.fillOval(0, 0, 179, 179);
        getBackground().drawImage(image, 400-image.getWidth()/2, 300-image.getHeight()/2);
    }
    
    // creates a bounded Level subclass world
    public Level(int w, int h, int c) { this(w, h, c, true); }
    
    // creates a Level subclass world
    public Level(int w, int h, int c, boolean b)
    {
        super(w, h, c, b);
        setPaintOrder(Player.class);
        setFields(); // gets map
        applyMap(); // applies map
        TextImage.setDefaultTextColor(new Color(255, 0, 0, 128)); // color for score and level actors
        ScoreControl.addToWorld(this, 80, 20); // the score display actor
        LevelControl.addToWorld(this, w-75, 20); // the level display actor
    }
    
    // applies a subclass level map (builds a world according to map data)
    private void applyMap()
    {
        int len = 0;
        for (int r=0; r<map.length; r++) for (int p=0; p<map[r].length(); p++)
        {
            Actor actor = null;
            int i = "wpo".indexOf(map[r].charAt(p));
            if (i == 0) // "w"all (block)
            { // to reducee number of actors, walls are extended horizontally when possible
                len++;
                if (p == map[r].length()-1 || 'w' != map[r].charAt(p+1)) // wall ends?
                {
                    actor = new Block(len);
                    addObject(actor, 30*(p+1)-actor.getImage().getWidth()/2, 15+30*r);
                    actor = null;
                    len = 0;
                }
            }     
            if (i == 1) actor = player; // "p"layer
            if (i == 2) actor = new Orb(); // "o"rb
            if (actor != null) addObject(actor, 15+30*p, 15+30*r);
        }
    }
    
    // if title screen and project is started, begin gamee
    public void started() { if ("Level".equals(getClass().getName())) startGame(); }
    public static void startGame() { Greenfoot.setWorld(new Level1()); }
    
    public void setFields(){} // to be overriden by subclasses to set fields added to this class
    
    /**
     * since all worlds are 'Level' type worlds, actors can access these methods easily,
     * regardless of the subtype of Level
     */
    public void nextLevel(){} // to be overriden by subclasses to proceed to next level
    public void resetLevel(){} // to be overriden by suclassees to reset the current level
    
    // added method to draw curved text 
    private void drawCircText(String s, Font font, Color tColor, Color bgColor,
        int style, boolean inverted, int x, int y, int dist, int angle, int rot)
    {
        for (int i=0; i<s.length(); i++)
        {
            String t = s.substring(i, i+1);
            TextImage text = new TextImage(t, font, tColor, bgColor).styled(style);
            BgInk ink = new BgInk();
            ink.setImage(text);
            addObject(ink, x, y);
            ink.turn(angle+i*rot);
            ink.move(dist);
            ink.turn(90);
            if (inverted) ink.turn(180);
            ink.apply();
        }
    }
}