/**
 * This class controls the level number and the actor that displays it in such a way that
 * the content is easily accessible by multiple worlds without the need to pass the data
 * between the worlds.
 * 
 * NOTE: this class, as currently written, must be used along with the TextImage class by
 * danpost (greenfoot.org username).  It can be modified to work without it by creating
 * GreenfootImage objects instead of TextImage objects.
 */
import greenfoot.*;
 
public final class LevelControl
{
    private static int level; // the level number
    private static Actor levelDisplay; // the level display actor
    
    // updates the image of the level display actor; creates the actor first if needed
    private static void updateDisplay()
    {
        // create actor if needed
        if (levelDisplay == null) levelDisplay = new LevelDisplay();
        // set image of actor
        levelDisplay.setImage(new TextImage("Level: "+level));
    }
    
    // adds display actor into a given world; can also be used to re-locate actor
    public static void addToWorld(World world, int x, int y)
    {
        updateDisplay();
        removeFromWorld(world);
        world.addObject(levelDisplay, x, y);
    }
    
    // removes display actor from a given world
    public static void removeFromWorld(World world)
    {
        if (levelDisplay.getWorld() == world) world.removeObject(levelDisplay);
    }
    
    // sets a value to the level number field; then updates the image of the level display actor
    public static void setLevel(int value)
    {
        level = value;
        updateDisplay();
    }
    
    // allows access to the value of the level number field
    public static int getLevel()
    {
        return level;
    }
 
    // class for an immovable actor that will display the level number
    private static class LevelDisplay extends Actor
    {
        public void setLocation(int x, int y) {}
    }
}