import greenfoot.*;
import java.awt.Color;

public class Player extends GQActor
{
    private Animation animation;
    
    public Player()
    {
        // the animation images
        GreenfootImage[] images = new GreenfootImage[64];
        for (int i=0; i<images.length; i++)
        {
            GreenfootImage image = new GreenfootImage(16, 16);
            image.setColor(new Color(128+127/(images.length-1)*i, 0, 0));
            image.fill();
            image.setColor(java.awt.Color.black);
            image.drawRect(0, 0, 15, 15);
            images[i] = image;
        }
        // the animation
        animation = new Animation(this, images);
        animation.setFrame(0);
        // the Q settings
        setHorizontalMovement("left", "right", 9*QVAL, 2);
        setGravity(60);
        setDragForces(20, 0);
        setObstacles(Block.class);
        setJumping("up", 10*QVAL);
    }

    public void act()
    {
        // movement
        moveVertically();
        if (hasJustJumped()) ScoreControl.add(1);
        moveHorizontally();
        // limit horizontal movement
        int wHalf = getImage().getWidth()/2;
        if (getX() <= wHalf)
        {
            if (getVX() < 0) setVX(0);
            setQLocation(wHalf*QVAL, getQY());
        }
        if (getX() >= getWorld().getWidth()-wHalf)
        {
            if (getVX() > 0) setVX(0);
            setQLocation((getWorld().getWidth()-wHalf)*QVAL, getQY());
        }
        // speed governed actions (speed increasees both number of particles created and brightens image)
        produceParticles(getVX(), getVY());
        animation.setFrame(((int)Math.abs(getVX())*animation.getFrames().length)/(getMaximumSpeed()+1));
        // death
        if (isTouching(Orb.class) || isTouching(Orb.Bullet.class))
        {
            getWorld().addObject(new GameOver("YOU\n\nLOST"), 300, 240);
            removeTouching(Orb.Bullet.class);
            getWorld().removeObject(this);
            Greenfoot.stop();
        }
    }
    
    private void produceParticles(int hs, int vs)
    {
        int angle = (int)(Math.atan2(-vs, -hs)*18000/Math.PI);
        int dist = (int)Math.hypot(hs, vs);
        for (int i=250; i<Math.abs(dist); i+=250) getWorld().addObject(new Particle(dist, angle), getX(), getY());
    }
}