/**
 * This class controls the score value and the actor that displays it in such a way that
 * the content is easily accessible by multiple worlds without the need to pass the data
 * between the worlds.
 * 
 * NOTE: this class, as currently written, must be used along with the TextImage class by
 * danpost (greenfoot.org username).  It can be modified to work without it by creating
 * GreenfootImage objects instead of TextImage objects.
 */
import greenfoot.*;
 
public final class ScoreControl
{
    private static int score; // the score field
    private static Actor scoreDisplay; // the display actor
    
    // updates the image of the display actor; creates a display actor first if needed
    private static void updateDisplay()
    {
        // create actor if needed
        if (scoreDisplay == null) scoreDisplay = new ScoreDisplay();
        // set image of actor
        scoreDisplay.setImage(new TextImage("Score: "+score));
    }
    
    // adds display actor into a given world; can also be used to re-locate actor
    public static void addToWorld(World world, int x, int y)
    {
        updateDisplay();
        removeFromWorld(world);
        world.addObject(scoreDisplay, x, y);
    }
    
    // removes display actor from a given world
    public static void removeFromWorld(World world)
    {
        if (scoreDisplay.getWorld() == world) world.removeObject(scoreDisplay);
    }
    
    // assigns a given value to the score field and updates the image of the display actor
    public static void setScore(int value)
    {
        score = value;
        updateDisplay();
    }
    
    // allows access to the value of the score field
    public static int getScore()
    {
        return score;
    }
    
    // allows a given amount to be added to the score value; updates the display actor image
    public static void add(int amount)
    {
        score += amount;
        updateDisplay();
    }
    
    // class for an immovable actor that will display the score value
    private static class ScoreDisplay extends Actor
    {
        public void setLocation(int x, int y) {}
    }
}